"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Highlighter = void 0;
const preact_1 = require("preact");
const model_1 = require("./model");
function getVNodeParent(vnode) {
    return (vnode._parent ||
        vnode.__ ||
        // Older Preact X versions used `__p`
        vnode.__p ||
        null);
}
function getDom(vnode) {
    return vnode._dom || vnode.__e || null;
}
let model = (0, model_1.newHighlight)();
const tracked = new Set();
const seen = new WeakMap();
let activeHighlighters = new Set();
setTimeout(() => {
    let oldDiffed = preact_1.options.diffed;
    let oldCommit = preact_1.options._commit || preact_1.options.__c;
    preact_1.options.diffed = (vnode) => {
        if (activeHighlighters.size > 0) {
            let p = vnode;
            let hasHighlightParent = false;
            while (p !== null) {
                const prev = seen.get(p);
                if (prev !== undefined) {
                    hasHighlightParent = prev;
                    break;
                }
                if (vnode.type === Highlighter) {
                    hasHighlightParent = true;
                    break;
                }
                p = getVNodeParent(p);
            }
            seen.set(vnode, hasHighlightParent);
            if (hasHighlightParent) {
                const dom = getDom(vnode);
                if (dom !== null && !tracked.has(dom)) {
                    tracked.add(dom);
                    model.update(dom);
                }
            }
        }
        if (oldDiffed)
            oldDiffed(vnode);
    };
    preact_1.options._commit = preact_1.options.__c = (vnode, queue) => {
        tracked.clear();
        if (oldCommit)
            oldCommit(vnode, queue);
    };
}, 500);
class Highlighter extends preact_1.Component {
    componentWillUnmount() {
        activeHighlighters.delete(this);
    }
    render() {
        activeHighlighters.add(this);
        return this.props.children;
    }
}
exports.Highlighter = Highlighter;
