import { Component, options } from "preact";
import { newHighlight } from "./model";
function getVNodeParent(vnode) {
    return (vnode._parent ||
        vnode.__ ||
        // Older Preact X versions used `__p`
        vnode.__p ||
        null);
}
function getDom(vnode) {
    return vnode._dom || vnode.__e || null;
}
let model = newHighlight();
const tracked = new Set();
const seen = new WeakMap();
let activeHighlighters = new Set();
setTimeout(() => {
    let oldDiffed = options.diffed;
    let oldCommit = options._commit || options.__c;
    options.diffed = (vnode) => {
        if (activeHighlighters.size > 0) {
            let p = vnode;
            let hasHighlightParent = false;
            while (p !== null) {
                const prev = seen.get(p);
                if (prev !== undefined) {
                    hasHighlightParent = prev;
                    break;
                }
                if (vnode.type === Highlighter) {
                    hasHighlightParent = true;
                    break;
                }
                p = getVNodeParent(p);
            }
            seen.set(vnode, hasHighlightParent);
            if (hasHighlightParent) {
                const dom = getDom(vnode);
                if (dom !== null && !tracked.has(dom)) {
                    tracked.add(dom);
                    model.update(dom);
                }
            }
        }
        if (oldDiffed)
            oldDiffed(vnode);
    };
    options._commit = options.__c = (vnode, queue) => {
        tracked.clear();
        if (oldCommit)
            oldCommit(vnode, queue);
    };
}, 500);
export class Highlighter extends Component {
    componentWillUnmount() {
        activeHighlighters.delete(this);
    }
    render() {
        activeHighlighters.add(this);
        return this.props.children;
    }
}
