import { Component, options, VNode } from "preact";
import { newHighlight } from "./model";

function getVNodeParent(vnode: VNode): VNode | null {
  return (
    (vnode as any)._parent ||
    (vnode as any).__ ||
    // Older Preact X versions used `__p`
    (vnode as any).__p ||
    null
  );
}
function getDom(vnode: VNode): HTMLElement | Text | null {
  return (vnode as any)._dom || (vnode as any).__e || null;
}

let model = newHighlight();

const tracked = new Set<HTMLElement>();
const seen = new WeakMap<VNode, boolean>();

let activeHighlighters = new Set<any>();

setTimeout(() => {
  let oldDiffed = options.diffed;
  let oldCommit = (options as any)._commit || (options as any).__c;

  options.diffed = (vnode) => {
    if (activeHighlighters.size > 0) {
      let p: VNode | null = vnode;
      let hasHighlightParent = false;
      while (p !== null) {
        const prev = seen.get(p);

        if (prev !== undefined) {
          hasHighlightParent = prev;
          break;
        }

        if (p.type === Highlighter) {
          hasHighlightParent = true;
          break;
        }
        p = getVNodeParent(p);
      }

      seen.set(vnode, hasHighlightParent);

      if (hasHighlightParent) {
        const dom = getDom(vnode);
        if (dom !== null && !tracked.has(dom as any)) {
          tracked.add(dom as HTMLElement);
          model.update(dom as HTMLElement);
        }
      }
    }

    if (oldDiffed) oldDiffed(vnode);
  };

  (options as any)._commit = (options as any).__c = (
    vnode: VNode,
    queue: any[]
  ) => {
    tracked.clear();
    if (oldCommit) oldCommit(vnode, queue);
  };
}, 500);

export class Highlighter extends Component<{ children: any }> {
  componentWillUnmount(): void {
    activeHighlighters.delete(this);
  }

  render() {
    activeHighlighters.add(this);
    return this.props.children;
  }
}
